@@ -115,10 +115,11 @@ int CV_DetectorTest::prepareData( FileStorage& _fs )
 //        fn[TOTAL_NO_PAIR_E] >> eps.totalNoPair;
 
         // read detectors
-        if( fn[DETECTOR_NAMES].size() != 0 )
+        FileNode fn_names = fn[DETECTOR_NAMES];
+        if( fn_names.size() != 0 )
         {
-            FileNodeIterator it = fn[DETECTOR_NAMES].begin();
-            for( ; it != fn[DETECTOR_NAMES].end(); )
+            FileNodeIterator it = fn_names.begin(), it_end = fn_names.end();
+            for( ; it != it_end; )
             {
                 String _name;
                 it >> _name;
@@ -137,7 +138,7 @@ int CV_DetectorTest::prepareData( FileStorage& _fs )
                 String filename;
                 it >> filename;
                 imageFilenames.push_back(filename);
-                Mat img = imread( dataPath+filename, 1 );
+                Mat img = imread( dataPath+filename, IMREAD_COLOR );
                 images.push_back( img );
             }
         }
@@ -357,6 +358,26 @@ int CV_DetectorTest::validate( int detectorIdx, vector<vector<Rect> >& objects )
         noPair += (int)count_if( map.begin(), map.end(), isZero );
         totalNoPair += noPair;
 
+        /*if( noPair > cvRound(valRects.size()*eps.noPair)+1 )
+        {
+            printf("Problem discovered: imageIdx = %d, cascade=%s: %d vs %d rects\n", imageIdx, detectorNames[detectorIdx].c_str(), (int)it->size(), (int)valRects.size());
+            Mat image = images[imageIdx].clone();
+            for( int k = 0; k < 2; k++ )
+            {
+                const std::vector<Rect>& imgObjects = k == 0 ? *it : valRects;
+                Scalar color = k == 0 ? Scalar(0, 255, 0) : Scalar(0, 0, 255);
+                for( size_t i = 0; i < imgObjects.size(); i++ )
+                {
+                    Rect r = imgObjects[i];
+                    rectangle(image, r, color, 3);
+                    if( k == 1 )
+                        putText(image, format("%d", (int)i), Point(r.x + r.width/4, r.y + r.height*3/4), FONT_HERSHEY_PLAIN, 2, Scalar(0, 0, 255), 3);
+                }
+            }
+            imshow("results", image);
+            waitKey();
+        }*/
+
         EXPECT_LE(noPair, cvRound(valRects.size()*eps.noPair)+1)
             << "detector " << detectorNames[detectorIdx] << " has overrated count of rectangles without pair on "
             << imageFilenames[imageIdx] << " image";
@@ -366,7 +387,7 @@ int CV_DetectorTest::validate( int detectorIdx, vector<vector<Rect> >& objects )
     }
 
     EXPECT_LE(totalNoPair, cvRound(totalValRectCount*eps./*total*/noPair)+1)
-        << "detector " << detectorNames[detectorIdx] << " has overrated count of rectangles without pair on all images set";
+        << "In total, detector " << detectorNames[detectorIdx] << " has overrated count of rectangles without pair on the whole image set";
 
     if (::testing::Test::HasFailure())
         return cvtest::TS::FAIL_BAD_ACCURACY;
@@ -383,7 +404,6 @@ protected:
     virtual void readDetector( const FileNode& fn );
     virtual void writeDetector( FileStorage& fs, int di );
     virtual int detectMultiScale( int di, const Mat& img, vector<Rect>& objects );
-    virtual int detectMultiScale_C( const string& filename, int di, const Mat& img, vector<Rect>& objects );
     vector<int> flags;
 };
 
@@ -413,36 +433,6 @@ void CV_CascadeDetectorTest::writeDetector( FileStorage& fs, int di )
     fs << C_SCALE_CASCADE << sc;
 }
 
-
-int CV_CascadeDetectorTest::detectMultiScale_C( const string& filename,
-                                                int di, const Mat& img,
-                                                vector<Rect>& objects )
-{
-    Ptr<CvHaarClassifierCascade> c_cascade(cvLoadHaarClassifierCascade(filename.c_str(), cvSize(0,0)));
-    Ptr<CvMemStorage> storage(cvCreateMemStorage());
-
-    if( !c_cascade )
-    {
-        ts->printf( cvtest::TS::LOG, "cascade %s can not be opened");
-        return cvtest::TS::FAIL_INVALID_TEST_DATA;
-    }
-    Mat grayImg;
-    cvtColor( img, grayImg, COLOR_BGR2GRAY );
-    equalizeHist( grayImg, grayImg );
-
-    CvMat c_gray = cvMat(grayImg);
-    CvSeq* rs = cvHaarDetectObjects(&c_gray, c_cascade, storage, 1.1, 3, flags[di] );
-
-    objects.clear();
-    for( int i = 0; i < rs->total; i++ )
-    {
-        Rect r = *(Rect*)cvGetSeqElem(rs, i);
-        objects.push_back(r);
-    }
-
-    return cvtest::TS::OK;
-}
-
 int CV_CascadeDetectorTest::detectMultiScale( int di, const Mat& img,
                                               vector<Rect>& objects)
 {
@@ -450,11 +440,6 @@ int CV_CascadeDetectorTest::detectMultiScale( int di, const Mat& img,
     filename = dataPath + detectorFilenames[di];
     const string pattern = "haarcascade_frontalface_default.xml";
 
-    if( filename.size() >= pattern.size() &&
-        strcmp(filename.c_str() + (filename.size() - pattern.size()),
-              pattern.c_str()) == 0 )
-        return detectMultiScale_C(filename, di, img, objects);
-
     CascadeClassifier cascade( filename );
     if( cascade.empty() )
     {
@@ -527,7 +512,7 @@ TEST(Objdetect_HOGDetectorReadWrite, regression)
     fs.open(tempfilename, FileStorage::READ);
     remove(tempfilename.c_str());
 
-    FileNode n = fs["opencv_storage"]["myHOG"];
+    FileNode n = fs["myHOG"];
 
     ASSERT_NO_THROW(hog.read(n));
 }
@@ -553,15 +538,15 @@ public:
         ts(cvtest::TS::ptr()), failed(false)
     { }
 
-    virtual void computeGradient(const Mat& img, Mat& grad, Mat& qangle,
+    virtual void computeGradient(InputArray img, InputOutputArray grad, InputOutputArray qangle,
         Size paddingTL, Size paddingBR) const;
 
-    virtual void detect(const Mat& img,
+    virtual void detect(InputArray img,
         vector<Point>& hits, vector<double>& weights, double hitThreshold = 0.0,
         Size winStride = Size(), Size padding = Size(),
         const vector<Point>& locations = vector<Point>()) const;
 
-    virtual void detect(const Mat& img, vector<Point>& hits, double hitThreshold = 0.0,
+    virtual void detect(InputArray img, vector<Point>& hits, double hitThreshold = 0.0,
         Size winStride = Size(), Size padding = Size(),
         const vector<Point>& locations = vector<Point>()) const;
 
@@ -985,7 +970,7 @@ inline bool HOGDescriptorTester::is_failed() const
 
 static inline int gcd(int a, int b) { return (a % b == 0) ? b : gcd (b, a % b); }
 
-void HOGDescriptorTester::detect(const Mat& img,
+void HOGDescriptorTester::detect(InputArray _img,
     vector<Point>& hits, vector<double>& weights, double hitThreshold,
     Size winStride, Size padding, const vector<Point>& locations) const
 {
@@ -996,6 +981,7 @@ void HOGDescriptorTester::detect(const Mat& img,
     if( svmDetector.empty() )
         return;
 
+    Mat img = _img.getMat();
     if( winStride == Size() )
         winStride = cellSize;
     Size cacheStride(gcd(winStride.width, blockStride.width),
@@ -1085,7 +1071,7 @@ void HOGDescriptorTester::detect(const Mat& img,
     }
 }
 
-void HOGDescriptorTester::detect(const Mat& img, vector<Point>& hits, double hitThreshold,
+void HOGDescriptorTester::detect(InputArray img, vector<Point>& hits, double hitThreshold,
     Size winStride, Size padding, const vector<Point>& locations) const
 {
     vector<double> weightsV;
@@ -1166,15 +1152,19 @@ void HOGDescriptorTester::compute(InputArray _img, vector<float>& descriptors,
     }
 }
 
-void HOGDescriptorTester::computeGradient(const Mat& img, Mat& grad, Mat& qangle,
+void HOGDescriptorTester::computeGradient(InputArray _img, InputOutputArray _grad, InputOutputArray _qangle,
    Size paddingTL, Size paddingBR) const
 {
+    Mat img = _img.getMat();
     CV_Assert( img.type() == CV_8U || img.type() == CV_8UC3 );
 
     Size gradsize(img.cols + paddingTL.width + paddingBR.width,
        img.rows + paddingTL.height + paddingBR.height);
-    grad.create(gradsize, CV_32FC2);  // <magnitude*(1-alpha), magnitude*alpha>
-    qangle.create(gradsize, CV_8UC2); // [0..nbins-1] - quantized gradient orientation
+    _grad.create(gradsize, CV_32FC2);  // <magnitude*(1-alpha), magnitude*alpha>
+    _qangle.create(gradsize, CV_8UC2); // [0..nbins-1] - quantized gradient orientation
+    Mat grad = _grad.getMat();
+    Mat qangle = _qangle.getMat();
+
     Size wholeSize;
     Point roiofs;
     img.locateROI(wholeSize, roiofs);
